﻿using System;
using System.Drawing;
using System.IO;
using System.Security.Cryptography.X509Certificates;

using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Graphics;
using GrapeCity.Documents.Pdf.AcroForms;
using GrapeCity.Documents.Pdf.Security;

namespace ConsoleApp1
{
    internal class Program
    {
        static void Main(string[] args)
        {
            GcPdfDocument doc = new GcPdfDocument();
            var p = doc.NewPage();

            // create an unsigned SignatureField with custom appearance
            SignatureField sf = new SignatureField();
            doc.AcroForm.Fields.Add(sf);
            sf.Widget.Page = p;
            sf.Widget.Rect = new RectangleF(20, 40, 200, 60);
            // create an appearance stream
            FormXObject fxo = sf.Widget.CreateAppearanceContentStream();
            GcGraphics g = fxo.Graphics;
            // draw appearance on fxoGraphics
            var r = fxo.Bounds;
            g.FillRectangle(r, Color.Yellow);
            TextFormat tf = new TextFormat()
            {
                FontName = "Tahoma",
                FontSize = 14,
            };
            g.DrawString("Some long text long text long text long text long text long text", tf, r, TextAlignment.Center, ParagraphAlignment.Center, true);
            // assign created appearance stream
            sf.Widget.AppearanceStreams.Normal.Default = fxo;
            doc.Save("unsigned.pdf");

            // Sign the document,
            // use CR LF to wrap long caption
            SignatureProperties sp = new SignatureProperties();
            sp.SignatureField = sf;
            sp.SignatureAppearance.TextFormat = new TextFormat()
            {
                FontName = "Tahoma",
                FontSize = 12,
            };
            var cert = new X509Certificate2(@"..\..\..\User.pfx", "User12");
            sp.SignatureBuilder = new Pkcs7SignatureBuilder(cert);
            sp.SignatureAppearance.Caption = "Some\r\nLong\r\nCaption";
            sp.SignatureAppearance.Image = Image.FromFile(@"..\..\..\exif.jpg");
            sp.SignatureAppearance.CaptionImageRelation = GrapeCity.Documents.Pdf.Annotations.CaptionImageRelation.CaptionAtRight;
            doc.Sign(sp, "signed.pdf", false);

            // Sign the document using custom appearance stream
            using (var fs = new FileStream("unsigned.pdf", FileMode.Open))
            {
                // load unsigned document
                doc.Load(fs);

                // get the SignatureField from the loaded document
                sf = doc.AcroForm.Fields[0] as SignatureField;
                // create and assign custom appearance stream to the signature field
                fxo = sf.Widget.CreateAppearanceContentStream();
                g = fxo.Graphics;
                r = fxo.Bounds;
                g.FillRectangle(r, Color.Yellow);
                tf = new TextFormat()
                {
                    FontName = "Tahoma",
                    FontSize = 14,
                };
                g.DrawString("SIGNED", tf, r, TextAlignment.Center, ParagraphAlignment.Center, true);
                sf.Widget.AppearanceStreams.Normal.Default = fxo;

                //sign the document
                sp = new SignatureProperties();
                sp.SignatureField = sf;
                // reset signature appearance so it will not be used to
                // generate an appearance stream
                sp.SignatureAppearance = null;
                sp.SignatureBuilder = new Pkcs7SignatureBuilder(cert);
                doc.Sign(sp, "signed2.pdf", false);
            }
        }
    }
}
